"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SsoConfigValidator = void 0;
const hostname_validator_1 = require("./hostname.validator");
class SsoConfigValidator {
    static validate(config) {
        const result = { ok: false };
        if (!config.ntlmHosts) {
            result.message = "Incomplete configuration. ntlmHosts is an required field.";
            return result;
        }
        if (!(config.ntlmHosts instanceof Array)) {
            result.message = "Invalid ntlmHosts, must be an array.";
            return result;
        }
        const allNtlmHostsPass = config.ntlmHosts.every((ntlmHost) => {
            if (!hostname_validator_1.HostnameValidator.validHostnameOrFqdn(ntlmHost)) {
                result.message =
                    "Invalid host [" +
                        hostname_validator_1.HostnameValidator.escapeHtml(ntlmHost) +
                        "] in ntlmHosts, must be only a hostname or FQDN " +
                        "(localhost or www.google.com is ok, https://www.google.com:443/search is not ok). Wildcards are accepted.";
                return false;
            }
            return true;
        });
        if (!allNtlmHostsPass) {
            return result;
        }
        result.ok = true;
        return result;
    }
}
exports.SsoConfigValidator = SsoConfigValidator;
